/*  configuration.c - functions to parse the configuration file
 *  Copyright (C) 2001 Kristof Verniers
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <mysql.h>
#include <stdlib.h>
#include <string.h>
#include "ts.h"

int readconfig(struct s_config *config)
{
  FILE* configfile;
  char line[MAX_CONFIGFILE_LINELENGTH];
  char *chr_ptr, *var = NULL, *value = NULL, *val_ptr = NULL;
  char *filename;
  int linenr = 0; 
  char equal;

  config->port = 3360;
  
  filename = strdup(getenv("HOME"));
  filename = (char *) realloc(filename, strlen(filename) + strlen("/.ts.conf") + 1);
  if (filename == NULL) {
    fprintf(stderr, "Unable to allocate enough memory.\n");
    return -1;
  }
  strcat(filename, "/.ts.conf");
  if ((configfile = fopen(filename, "r")) == NULL) {
    free(filename);
    filename = (char *) malloc(strlen("/etc/ts.conf")+1);
    strcpy(filename, "/etc/ts.conf");
    if ((configfile = fopen(filename, "r")) == NULL) {
      printf("I couldn't open neither ~/.ts.conf, nor /etc/ts.conf.\n");
      printf("Please create at least one of both files.\n");
      free(filename);
      return -1;
    }
  }

  while (fgets(line, MAX_CONFIGFILE_LINELENGTH, configfile)) {
    chr_ptr = line;
    ++linenr;
    // eat whitespace
    while ((*chr_ptr != '\0') && ((*chr_ptr == ' ') || (*chr_ptr == '\t'))) ++chr_ptr;
    // continue if no blank line or comment line
    if ((*chr_ptr != '\n') && (*chr_ptr != '#')) {
      if ((sscanf(chr_ptr, "%as %c %as", &var, &equal, &value) < 3) || (equal != '=')) {
        printf("Parse error in configuration file '%s' on line %i: %s", filename, linenr, line);
        printf("A legal entry in this file is of the form 'var = value' (leave at least one whitespace on both sites of the =.\n");
        fclose(configfile);
        free(filename);
        return -1;
      }
      val_ptr = value;
      // remove "s
      if ((*val_ptr == '"') && (strlen(val_ptr) > 1) && (*(val_ptr+strlen(val_ptr)-1) == '"')) {
        ++val_ptr;
        *(val_ptr+strlen(val_ptr)-1) = '\0';
      }
      if (strcmp(var, "host") == 0) {
        config->host = strdup(val_ptr);
      } else if (strcmp(var, "user") == 0) {
        config->user = strdup(val_ptr);
      } else if (strcmp(var, "passwd") == 0) {
        config->passwd = strdup(val_ptr);
      } else if (strcmp(var, "db") == 0) {
        config->db = strdup(val_ptr);
      } else if (strcmp(var, "port") == 0) {
        config->port = atoi(val_ptr);
        if (!(config->port > 0)) {
          printf("Parse error in configuration file '%s' on line %i: %s", filename, linenr, line);
          printf("The portnumber should be an integer value > 0.\n");
          free(var);
          free(value);
          free(filename);
          fclose(configfile);
          return -1;
        }
      } else {
        printf("Parse error in configuration file '%s' on line %i: %s", filename, linenr, line);
        printf("'%s' is an unknown configuration variable.\n", var);
        free(var);
        free(value);
        free(filename);
        fclose(configfile);
        return -1;
      }
      free(var);
      free(value);
    }
  }

  fclose(configfile);
  
  if (config->host == NULL) {
    printf("Please specify the hostname of the mysql server in the configuration file '%s'.\n", filename);
    return -1;
  }
  if (config->user == NULL) {
    printf("Please specify a username for the mysql server in the configuration file '%s'.\n", filename);
    return -1;
  }
  if (config->passwd == NULL) {
    printf("Please specify the password for the mysql server in the configuration file '%s'.\n", filename);
    return -1;
  }
  if (config->db == NULL) {
    printf("Please specify the name of the database in the configuration file '%s'.\n", filename);
    return -1;
  }

  free(filename);
  return 1;

}

