/*  projects.c - Functions for dealing with projects.
 *  Copyright (C) 2001 Kristof Verniers
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <mysql.h>
#include <stdio.h>
#include <stdlib.h>
#include "ts.h"

int run_projects_query(MYSQL *mysql, char *query)
{
  MYSQL_ROW row;
  MYSQL_RES *result;
  int count = 0;
  int ok = TRUE;
  char prev_cust[MAX_KEYLENGTH+1];
  int first = TRUE;
  char *cust_abbrev, *cust_name, *proj_abbrev, *proj_name, *spent, *started;
  long spenttime;

  if (mysql_query(mysql, query)) {
    fprintf(stderr, "Problem while running query: %s.\n", mysql_error(mysql));
    return FAILURE;
  }

  result = mysql_use_result(mysql);

  // read in result
  strcpy(prev_cust, "");
  while ((row = mysql_fetch_row(result))) {
    cust_name = *row;
    cust_abbrev = *++row;
    proj_name = *++row;
    proj_abbrev = *++row;
    spent = *++row;
    started = *++row;
    if (strcmp(cust_abbrev, prev_cust) != 0) {
      if (!first) printf("\n");
      printf("%s (%s)\n", cust_name, cust_abbrev);
      strcpy(prev_cust, cust_abbrev);
      first = FALSE;
    }
    printf("- %s (%s)", proj_name, proj_abbrev);
    spenttime = atol(spent);
    printf("   --> %0.2f hours or %0.2f days", spenttime / 3600.0, spenttime / 3600.0 / 8.0);
    if ((started != NULL) && strcmp("0", started) == 0) printf(" - **** STARTED ****");
    printf("\n");
    ++count;
  }
  if (mysql_errno(mysql)) {
    fprintf(stderr, "Error: %s\n", mysql_error(mysql));
    ok = FALSE;
  } else {
    if (!count) printf("No projects found.\n");
  }

  // free resultset
  mysql_free_result(result);

  return (ok ? SUCCESS : FAILURE);

}

int list_projects(int argc, char *argv[])
{
  MYSQL mysql;
  int result;
  char query[1000];
  char project[MAX_KEYLENGTH+1];
  char customer[MAX_KEYLENGTH+1];

  if (argc > 2) {
    syntax(HELP_LIST_PROJECTS);
    return FAILURE;
  }

  if (!get_db_connection(&mysql)) return FAILURE;

  if (argc == 0) {
    strcpy(query, "select customers.name as customername, projects.customer as customerabbrev, \
		    projects.name as projectname, projects.abbrev as projectabbrev, \
		    sum((if(stop = 0, unix_timestamp(), stop))-start) as spenttime, \
		    min(stop) as started from projects left join timesheets on \
		    timesheets.project = projects.abbrev and timesheets.customer = \
		    projects.customer inner join customers on customers.abbrev = \
		    projects.customer group by customerabbrev, projectabbrev order \
		    by customername, projectname");
  }
  if (argc == 1) {
    prepare(&mysql, customer, *argv, MAX_KEYLENGTH);
    sprintf(query, "select customers.name as customername, projects.customer as customerabbrev, \
		    projects.name as projectname, projects.abbrev as projectabbrev, \
		    sum((if(stop = 0, unix_timestamp(), stop))-start) as spenttime, \
		    min(stop) as started from projects left join timesheets on \
		    timesheets.project = projects.abbrev and timesheets.customer = \
		    projects.customer inner join customers on customers.abbrev = \
		    projects.customer and customers.abbrev =  '%s' group by projectabbrev \
		    order by customername, projectname", customer);
  }
  if (argc == 2) {
    prepare(&mysql, customer, *argv++, MAX_KEYLENGTH);
    prepare(&mysql, project, *argv, MAX_KEYLENGTH);
    sprintf(query, "select customers.name as customername, projects.customer as customerabbrev, \
		    projects.name as projectname, projects.abbrev as projectabbrev, \
		    sum((if(stop = 0, unix_timestamp(), stop))-start) as spenttime, \
		    min(stop) as started from projects left join timesheets on \
		    timesheets.project = projects.abbrev and timesheets.customer = \
		    projects.customer inner join customers on customers.abbrev = \
		    projects.customer and customers.abbrev = '%s' and projects.abbrev = \
		    '%s' group by projectabbrev order by customername, projectname", 
		    customer, project);
  }

  result = run_projects_query(&mysql, query);

  close_db_connection(&mysql);
  
  return SUCCESS;
}

int add_project(int argc, char *argv[])
{
  MYSQL mysql;
  int result;
  char customer_key[MAX_KEYLENGTH+1];
  char project_key[MAX_KEYLENGTH+1];
  char name[MAX_TEXTLENGTH+1];
  char query[1000];

  if (argc != 3) {
    syntax(HELP_ADD_PROJECT);
    return FAILURE;
  }

  if (!get_db_connection(&mysql)) return FAILURE;

  prepare(&mysql, customer_key, *argv++, MAX_KEYLENGTH);
  prepare(&mysql, project_key, *argv++, MAX_KEYLENGTH);
  prepare(&mysql, name, *argv, MAX_TEXTLENGTH);

  // does the customer exist?
  sprintf(query, "select * from customers where abbrev = '%s'", customer_key);
  if (!returns_rows(&mysql, query)) {
    printf("Please specify an existing customer.\n");
    close_db_connection(&mysql);
    return FAILURE;
  }
  
  // insert the project
  sprintf(query, "insert into projects (customer, abbrev, name) values ('%s', '%s', '%s')", customer_key, project_key, name); 
  result = mysql_query(&mysql, query);
  if (result != 0) {
    printf("Unable to create this project. Did you specify a unique key?\n");
  } else {
    printf("Project added.\n");
  }
  
  close_db_connection(&mysql);
  
  return (result == 0 ? SUCCESS : FAILURE);
	
}

int del_project(int argc, char *argv[])
{
  MYSQL mysql;
  int result;
  char customer_key[MAX_KEYLENGTH+1];
  char project_key[MAX_KEYLENGTH+1];
  char query[1000];
	
  if (argc != 2) {
    syntax(HELP_DEL_PROJECT);
    return FAILURE;
  }

  if (!get_db_connection(&mysql)) return FAILURE;

  prepare(&mysql, customer_key, *argv++, MAX_KEYLENGTH);
  prepare(&mysql, project_key, *argv, MAX_KEYLENGTH);
 
  sprintf(query, "delete from timesheets where customer = '%s' and project = '%s'", customer_key, project_key);
  result = mysql_query(&mysql, query);
  if (result != 0) {
    printf("Unable to delete the timesheets for this project: %s\n", mysql_error(&mysql));
    close_db_connection(&mysql);
    return FAILURE;
  }
  
  sprintf(query, "delete from projects where customer = '%s' and abbrev = '%s'", customer_key, project_key);
  result = mysql_query(&mysql, query);
  if (result != 0) {
    printf("Unable to delete the projects: %s\n", mysql_error(&mysql));
    close_db_connection(&mysql);
    return FAILURE;
  }
  
  if (mysql_affected_rows(&mysql) < 1) {
    printf("No projects deleted.\n");
  } else {
    printf("Project deleted.\n");
  }
  
  close_db_connection(&mysql);

  return SUCCESS;
	
}


