/*  timesheets.c - Functions for dealing with timesheets.
 *  Copyright (C) 2001 Kristof Verniers
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <mysql.h>
#include <stdio.h>
#include <stdlib.h>
#include "ts.h"

int list_timesheets(int argc, char *argv[])
{
  MYSQL mysql;
  MYSQL_ROW row;
  MYSQL_RES *result;
  char customer_key[MAX_KEYLENGTH+1];
  char project_key[MAX_KEYLENGTH+1];
  char query[1000];
  char *rec_id, *starttime, *stoptime, *spent, *finished, *remark;
  long spenttime;
  long totaltime = 0;

  if (argc != 2) {
    syntax(HELP_LIST_TIMESHEETS);
    return FAILURE;
  }

  if (!get_db_connection(&mysql)) return FAILURE;

  prepare(&mysql, customer_key, *argv++, MAX_KEYLENGTH);
  prepare(&mysql, project_key, *argv, MAX_KEYLENGTH);

  sprintf(query, "select rec_id, from_unixtime(start) as starttime, 
		  from_unixtime(stop) as stoptime, ((if(stop = 0, 
                  unix_timestamp(), stop))-start) as spent, if(stop = 0, 
                  0, 1) as finished, remark from timesheets where customer 
		  = '%s' and project = '%s' order by starttime asc",
		  customer_key, project_key);
  if (mysql_query(&mysql, query)) {
    fprintf(stderr, "Problem while running query: %s.\n", mysql_error(&mysql));
    return FAILURE;
  }

  result = mysql_store_result(&mysql);

  // read in result
  while ((row = mysql_fetch_row(result))) {
    rec_id = *row++;
    starttime = *row++;
    stoptime = *row++;
    spent = *row++;
    finished = *row++;
    remark = *row;
    spenttime = atol(spent);
    
    print_padded(rec_id, 4);
    printf(". ");
    print_padded(starttime, 19);
    printf(" -> ");
    if (strcmp(finished, "0") == 0) print_padded("...", 19) ;
      else print_padded(stoptime, 19); 
    printf("  = %3.2f hours or %2.2f days", spenttime/3600.0, spenttime/3600.0/8.0);
    if (strlen(remark) > 0) printf(" (%s)", remark);
    printf("\n");
    totaltime += spenttime;
  }
  
  if (mysql_num_rows(result) > 0) {
    printf("\n   Total time: %3.2f hours or %2.2f days.\n", totaltime/3600.0, totaltime/3600.0/8.0);
  } else {
    printf("No timesheets found.\n");
  }

  free(result);

  close_db_connection(&mysql);

  return SUCCESS;
}

int del_timesheet(int argc, char *argv[])
{
  MYSQL mysql;
  int result;
  char timesheet_key[MAX_KEYLENGTH+1];
  char query[1000];
	
  if (argc != 1) {
    syntax(HELP_DEL_TIMESHEET);
    return FAILURE;
  }

  if (!get_db_connection(&mysql)) return FAILURE;

  prepare(&mysql, timesheet_key, *argv++, MAX_KEYLENGTH);
 
  sprintf(query, "delete from timesheets where rec_id = '%s'", timesheet_key);
  result = mysql_query(&mysql, query);
  if (result != 0) {
    printf("Unable to delete this timesheet.\n");
    close_db_connection(&mysql);
    return FAILURE;
  }
  
  if (mysql_affected_rows(&mysql) < 1) {
    printf("No timesheets deleted.\n");
  } else {
    printf("Timesheet deleted.\n");
  }
  
  close_db_connection(&mysql);

  return SUCCESS;
}

int start_timesheet(int argc, char *argv[])
{
  MYSQL mysql;
  int result;
  char customer_key[MAX_KEYLENGTH+1];
  char project_key[MAX_KEYLENGTH+1];
  char query[1000];
	
  if (argc != 2) {
    syntax(HELP_START);
    return FAILURE;
  }

  if (!get_db_connection(&mysql)) return FAILURE;

  prepare(&mysql, customer_key, *argv++, MAX_KEYLENGTH);
  prepare(&mysql, project_key, *argv, MAX_KEYLENGTH);
 
  // check whether project exists
  sprintf(query, "select * from projects where customer = '%s' and abbrev = '%s'", customer_key, project_key);
  if (!returns_rows(&mysql, query)) {
    printf("Please specify an existing project.\n");
    close_db_connection(&mysql);
    return FAILURE;
  }
  
  // check whether project doesn't have a started timesheet
  sprintf(query, "select * from timesheets where customer = '%s' and project = '%s' and stop = 0", customer_key, project_key);
  if (returns_rows(&mysql, query)) {
    printf("This project currently has a started timesheet. Please stop this timesheet first.\n");
    close_db_connection(&mysql);
    return FAILURE;
  }

  // insert timesheet
  sprintf(query, "insert into timesheets (customer, project, start) values ('%s', '%s', unix_timestamp())", customer_key, project_key); 
  result = mysql_query(&mysql, query);
  if (result != 0) {
    printf("Unable to create a new timesheet: %s.\n", mysql_error(&mysql));
  } else {
    printf("Timesheet started.\n");
  }
  
  close_db_connection(&mysql);

  return SUCCESS;
}

int stop_timesheet(int argc, char *argv[])
{
  MYSQL mysql;
  int result;
  char customer_key[MAX_KEYLENGTH+1];
  char project_key[MAX_KEYLENGTH+1];
  char remark[MAX_TEXTLENGTH+1];
  char query[1000];

  if ((argc < 2) || (argc > 3)) {
    syntax(HELP_STOP);
    return FAILURE;
  }

  if (!get_db_connection(&mysql)) return FAILURE;

  prepare(&mysql, customer_key, *argv++, MAX_KEYLENGTH);
  prepare(&mysql, project_key, *argv++, MAX_KEYLENGTH);
  if (argc == 3) prepare(&mysql, remark, *argv, MAX_TEXTLENGTH);
    else strcpy(remark, "");
 
  // check whether project exists
  sprintf(query, "select * from projects where customer = '%s' and abbrev = '%s'", customer_key, project_key);
  if (!returns_rows(&mysql, query)) {
    printf("Please specify an existing project.\n");
    close_db_connection(&mysql);
    return FAILURE;
  }
  
  // update open timesheet
  sprintf(query, "update timesheets set stop = unix_timestamp(), 
		  remark = '%s' where customer = '%s' and 
		  project = '%s' and stop = 0", remark, customer_key, project_key);
  result = mysql_query(&mysql, query);
  if (result != 0) {
    printf("Unable to stop an open timesheet: %s.\n", mysql_error(&mysql));
  } else {
    switch (mysql_affected_rows(&mysql)) {
      case 0: printf("No started timesheet was found for this project.\n"); break;
      case 1: printf("Timesheet stopped.\n"); break;
      default: printf("More than one started timesheet was found for the\n");
               printf("project. This situation normally shouldn't occur.\n");
               printf("The stoptime of all of these timesheets was set to the\n");
               printf("current time. Please check your timesheets for this project\n");
               printf("manually.\n"); break;
    };
  }
  
  close_db_connection(&mysql);

  return SUCCESS;
}

int modify_timesheet(int argc, char *argv[])
{
  MYSQL mysql;
  MYSQL_ROW row;
  MYSQL_RES *result;
  char timesheet_key[MAX_KEYLENGTH+1];
  char newdate[25];
  long lng_date;
  char field[11];
  char remark[MAX_TEXTLENGTH+1];
  char query[1000];
  long starttime, stoptime;
  int intresult;

  if (argc != 3) {
    syntax(HELP_MODIFY_TIMESHEET);
    return FAILURE;
  }
  
  if (!get_db_connection(&mysql)) return FAILURE;

  prepare(&mysql, timesheet_key, *argv++, MAX_KEYLENGTH);
  prepare(&mysql, field, *argv++, 10);

  if (!((strcmp(field, "start") == 0) || (strcmp(field, "stop") == 0) || (strcmp(field, "remark") == 0))) {
    syntax(HELP_MODIFY_TIMESHEET);
    close_db_connection(&mysql);
    return FAILURE;
  }

  // get start and stop-time 
  sprintf(query, "select start, stop from timesheets where rec_id = '%s'", timesheet_key);
  if (mysql_query(&mysql, query)) {
    fprintf(stderr, "Problem while running query: %s.\n", mysql_error(&mysql));
    close_db_connection(&mysql);
    return FAILURE;
  }
  result = mysql_store_result(&mysql);
  if ((row = mysql_fetch_row(result))) {
    starttime = atol(*row++); 
    stoptime = atol(*row); 
    free(result);
  } else {
    printf("No timesheet with that id.\n");
    free(result);
    close_db_connection(&mysql);
    return FAILURE;
  }

  // build the query to update the timesheet
  strcpy(query, "");
  if (strcmp(field, "start") == 0) {
    prepare(&mysql, newdate, *argv, 24);
    lng_date = get_date(&mysql, newdate);
    if (!(lng_date > 0)) {
      printf("Please specify a valid date of the form 'YYYY-MM-DD HH:MM:SS'.\n");
      close_db_connection(&mysql);
      return FAILURE;
    }
    if (!(stoptime > lng_date)) {
      printf("Please make sure that starttime < stoptime.\n");
      close_db_connection(&mysql);
      return FAILURE;
    }
    sprintf(query, "update timesheets set start = %ld where rec_id = '%s'", lng_date, timesheet_key);
  } else if (strcmp(field, "stop") == 0) {
    prepare(&mysql, newdate, *argv, 24);
    lng_date = get_date(&mysql, newdate);
    if (!(lng_date > 0)) {
      printf("Please specify a valid date of the form 'YYYY-MM-DD HH:MM:SS'.\n");
      close_db_connection(&mysql);
      return FAILURE;
    }
    if (!(lng_date > starttime)) {
      printf("Please make sure that stoptime > starttime.\n");
      close_db_connection(&mysql);
      return FAILURE;
    }
    sprintf(query, "update timesheets set stop = %ld where rec_id = '%s'", lng_date, timesheet_key);
  } else if (strcmp(field, "remark") == 0) {
    prepare(&mysql, remark, *argv, MAX_TEXTLENGTH);
    sprintf(query, "update timesheets set remark = '%s' where rec_id = '%s'", remark, timesheet_key);
  }
  // execute the query
  intresult = mysql_query(&mysql, query);
  if (intresult != 0) {
    printf("Unable to update the timesheet: %s.\n", mysql_error(&mysql));
    close_db_connection(&mysql);
    return FAILURE;
  } else {
    printf("Timesheet updated.\n");
  }
   
  close_db_connection(&mysql);
  return SUCCESS;
}

